@@ -28,12 +28,15 @@ extern "C" {
 #endif
 
 /**
-  * \defgroup scalar extraction functions
+  * \defgroup scalar scalar extraction functions
+  * 
+  * Functions that extract a feature as a single value from an input vector
   *
-  * Defines scalar extraction functions, and their parameters.
   * @{
   */
 
+void test(void);
+
 /** \brief Extract the mean of an input vector
  * 
  * \param *data: a pointer to the first element 
@@ -70,6 +73,7 @@ int xtract_standard_deviation(const float *data, const int N, const void *argv,
  */
 int xtract_average_deviation(const float *data, const int N, const void *argv, float *result);
 
+
 /** \brief Extract the skewness of an input vector
  * 
  * \param *data: a pointer to the first element in an array of floats
@@ -123,7 +127,9 @@ int xtract_spectral_standard_deviation(const float *data, const int N, const voi
  * \param *argv: a pointer to a float representing the spectral mean of the input spectrum
  * \param *result: the  average deviation of the spectrum pointed to by *data
  */
+/*
 int xtract_spectral_average_deviation(const float *data, const int N, const void *argv, float *result);
+*/
 
 /** \brief Extract the skewness of an input spectrum
  * 
@@ -200,7 +206,7 @@ int xtract_smoothness(const float *data, const int N, const void *argv, float *r
  * 
  * \param *data: a pointer to the first element in an array of floats representing the magnitude coefficients from the spectrum of an audio vector, (e.g. the first half of the array pointed to by *result from xtract_spectrum().
  * \param N: the number of elements to be considered
- * \param *argv: a pointer to NULL
+ * \param *argv: a pointer to a float corresponding to the spectral centroid 
  * \param *result: the spectral spread of N values from the array pointed to by *data
  */
 int xtract_spread(const float *data, const int N, const void *argv, float *result);
@@ -237,21 +243,34 @@ int xtract_rolloff(const float *data, const int N, const void *argv, float *resu
  */
 int xtract_loudness(const float *data, const int N, const void *argv, float *result);
 
-/** \brief Extract the spectral flatness measure of an input vector using a method described by Tristan Jehan (2005)
+/** \brief Extract the spectral flatness measure of an input vector, where the flatness measure (SFM) is defined as the ratio of the geometric mean to the arithmetic mean of a magnitude spectrum.
+ *
+ * \note The computation method used here is the most efficient by a significant margin, but suffers from precision problems due to the multiplication operationin the geometric mean calculation. This is particularly accute for larger values of N (>=256). However, as noted by Peeters (2003), the SFM should generally be computed on a small number of 'bands' rather than on the complete magnitude spectrum. It is therefore highly recommended that xtract_bands() is used prior to calling xtract_flatness().
  * 
- * \param *data: a pointer to the first element in an array of floats representing the magnitude coefficients from the spectrum of an audio vector, (e.g. the first half of the array pointed to by *result from xtract_spectrum().
- * \param N: the number of elements to be considered
+ * \param *data: a pointer to the first element in an array of floats representing the magnitude coefficients from the spectrum of an audio vector, (e.g. the first half of the array pointed to by *result from xtract_spectrum(). Alternatively the magnitudes from a number of 'subbands' can be used by using *result from xtract_bands().
+ * \param N: the number of *data array elements to be considered
  * \param *argv: a pointer to NULL
- * \param *result: the spectral flatness of N values from the array pointed to by *data
+ * \param *result: the flatness of N values from the array pointed to by *data
  */
 int xtract_flatness(const float *data, const int N, const void *argv, float *result);
 
+/** \brief Extract the LOG spectral flatness measure of an input vector
+ *
+ * \param *data: a pointer to NULL.
+ * \param N: not used - can safely be set to 0.
+ * \param *argv: a pointer to a float represnting spectral flatness.
+ * \param *result: the LOG spectral flatness of N values from the array pointed to by *data
+ *
+ *  flatness_db = 10 * log10(flatness)
+ *
+ */
+int xtract_flatness_db(const float *data, const int N, const void *argv, float *result);
 
-/** \brief Extract the tonality factor of an input vector using a method described by Tristan Jehan (2005)
+/** \brief Extract the tonality factor of an input vector using a method described by Peeters 2003
  * 
- * \param *data: not used.
- * \param N: not used
- * \param *argv: a pointer to the spectral flatness measure of an audio vector (e.g. the output from xtract_flatness)
+ * \param *data: a pointer to NULL.
+ * \param N: not used - can safely be set to 0.
+ * \param *argv: a pointer to the LOG spectral flatness measure of an audio vector (e.g. the output from xtract_flatness_db)
  * \param *result: the tonality factor of N values from the array pointed to by *data
  */
 int xtract_tonality(const float *data, const int N, const void *argv, float *result);
@@ -307,7 +326,7 @@ int xtract_power(const float *data, const int N, const void *argv, float *result
  * \param *data: a pointer to the first element in an array of floats representing the amplitudes of the harmonic spectrum of an audio vector. It is sufficient to pass in a pointer to the first half of the array pointed to by *result from xtract_harmonic_spectrum().
  * \param N: the number of elements to be considered. If using the array pointed to by *result from xtract_harmonics, N should equal half the total array size i.e., just the amplitudes of the peaks.
  * \param *argv: a pointer to NULL
- * \param *result: the odd/even harmonic ratio of N values from the array pointed to by *data
+ * \param *result: the even/odd harmonic ratio of N values from the array pointed to by *data
  */
 int xtract_odd_even_ratio(const float *data, const int N, const void *argv, float *result);
 
@@ -378,6 +397,8 @@ int xtract_hps(const float *data, const int N, const void *argv, float *result);
  * \param *result: the pitch of N values from the array pointed to by *data
  *
  * This algorithm is based on the AMDF, with peak and centre clipping. It would benefit from further improvements to improve noise robustness and overall efficiency
+ *
+ * It is based on suggestion by Robert Bristow-Johnson in a discussion on the comp.dsp mailing list, subject "Reference implementation of pitch detection"
  * 
  */
 int xtract_f0(const float *data, const int N, const void *argv, float *result);
